Package: decompTumor2Sig
Type: Package
Title: Decomposition of individual tumors into mutational signatures by
        signature refitting
Depends: R(>= 4.0), ggplot2
Imports: methods, Matrix, quadprog(>= 1.5-5), GenomicRanges, stats,
        GenomicFeatures, Biostrings, BiocGenerics, S4Vectors, plyr,
        utils, graphics, BSgenome.Hsapiens.UCSC.hg19,
        TxDb.Hsapiens.UCSC.hg19.knownGene, VariantAnnotation,
        SummarizedExperiment, ggseqlogo, gridExtra, data.table,
        Seqinfo, readxl
Suggests: knitr, rmarkdown, BiocStyle
Version: 2.28.0
Date: 2025-07-22
Encoding: UTF-8
Authors@R: c(
    person("Rosario M.", "Piro", email = "rmpiro@gmail.com",
            role = c("aut", "cre")),
    person("Sandra", "Krueger", role = c("ctb"))
    )
Description: Uses quadratic programming for signature refitting, i.e., to
    decompose the mutation catalog from an individual tumor sample into a set of
    given mutational signatures (either Alexandrov-model signatures or
    Shiraishi-model signatures), computing weights that reflect the
    contributions of the signatures to the mutation load of the tumor.
License: GPL-2
URL: http://rmpiro.net/decompTumor2Sig/,
        https://github.com/rmpiro/decompTumor2Sig
BugReports: https://github.com/rmpiro/decompTumor2Sig/issues
biocViews: Software, SNP, Sequencing, DNASeq, GenomicVariation,
        SomaticMutation, BiomedicalInformatics, Genetics,
        BiologicalQuestion, StatisticalMethod
VignetteBuilder: knitr
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/decompTumor2Sig
git_branch: RELEASE_3_23
git_last_commit: 83dd5b2
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:40:33 UTC; biocbuild
Author: Rosario M. Piro [aut, cre],
  Sandra Krueger [ctb]
Maintainer: Rosario M. Piro <rmpiro@gmail.com>
