# decompTumor2Sig
decompTumor2Sig - Decomposition of individual tumors into mutational
signatures by signature refitting

Please visit the package's Bioconductor website for more information:

https://www.bioconductor.org/packages/release/bioc/html/decompTumor2Sig.html

