.onAttach <- function(libname, pkgname){
     packageStartupMessage( "" )
     packageStartupMessage( "Welcome to decompTumor2Sig" )
     packageStartupMessage( "--------------------------" )
     packageStartupMessage( "" )
     packageStartupMessage( "See ?decompTumor2Sig for a list of" )
     packageStartupMessage( "available functions." )
     packageStartupMessage( "" )
}
